@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2006 Kenneth J. Pronovici.
+# Copyright (c) 2006-2007 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Official Cedar Backup Extensions
-# Revision : $Id: mbox.py 1020 2006-12-19 04:40:08Z pronovic $
+# Revision : $Id: mbox.py 1104 2007-02-17 16:51:19Z pronovic $
 # Purpose  : Provides an extension to back up mbox email files.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -100,13 +100,12 @@ from gzip import GzipFile
 
 # Cedar Backup modules
 from CedarBackup2.filesystem import FilesystemList, BackupFileList
-from CedarBackup2.config import createInputDom, addContainerNode, addStringNode
-from CedarBackup2.config import isElement, readChildren, readFirstChild, readString
+from CedarBackup2.xmlutil import createInputDom, addContainerNode, addStringNode
+from CedarBackup2.xmlutil import isElement, readChildren, readFirstChild, readString, readStringList
 from CedarBackup2.config import VALID_COLLECT_MODES, VALID_COMPRESS_MODES
-from CedarBackup2.action import isStartOfWeek, buildNormalizedPath
+from CedarBackup2.util import isStartOfWeek, buildNormalizedPath
 from CedarBackup2.util import resolveCommand, executeCommand
 from CedarBackup2.util import ObjectTypeList, UnorderedList, encodePath, changeOwnership
-from CedarBackup2.xmlutil import readStringList, readString
 
 
 ########################################################################
@@ -1079,7 +1078,7 @@ def executeAction(configPath, options, config):
          collectMode = _getCollectMode(local, mboxDir)
          compressMode = _getCompressMode(local, mboxDir)
          lastRevision = _loadLastRevision(config, mboxDir, fullBackup, collectMode)
-         (excludePaths, excludePatterns) = _getExclusions(config, mboxDir)
+         (excludePaths, excludePatterns) = _getExclusions(mboxDir)
          if fullBackup or (collectMode in ['daily', 'incr', ]) or (collectMode == 'weekly' and todayIsStart):
             logger.debug("Mbox directory meets criteria to be backed up today.")
             _backupMboxDir(config, mboxDir.absolutePath, 
@@ -1198,7 +1197,7 @@ def _writeNewRevision(config, item, newRevision):
    except:
       logger.error("Failed to write revision file [%s] to disk." % revisionPath)
 
-def _getExclusions(config, mboxDir):
+def _getExclusions(mboxDir):
    """
    Gets exclusions (file and patterns) associated with an mbox directory.
 
@@ -1210,7 +1209,6 @@ def _getExclusions(config, mboxDir):
    backup for a given directory.  It is derived from the mbox directory's list
    of patterns.
 
-   @param config: Cedar Backup configuration.
    @param mboxDir: Mbox directory object.
 
    @return: Tuple (files, patterns) indicating what to exclude.
