@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2004-2006 Kenneth J. Pronovici.
+# Copyright (c) 2004-2007 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: filesystem.py 1012 2006-12-18 23:52:27Z pronovic $
+# Revision : $Id: filesystem.py 1127 2007-02-18 07:55:02Z pronovic $
 # Purpose  : Provides filesystem-related objects.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -57,7 +57,8 @@ import tarfile
 
 # Cedar Backup modules
 from CedarBackup2.knapsack import firstFit, bestFit, worstFit, alternateFit
-from CedarBackup2.util import AbsolutePathList, ObjectTypeList, UnorderedList, calculateFileAge, encodePath
+from CedarBackup2.util import AbsolutePathList, ObjectTypeList, UnorderedList
+from CedarBackup2.util import removeKeys, displayBytes, calculateFileAge, encodePath
 
 
 ########################################################################
@@ -84,10 +85,10 @@ class FilesystemList(list):
    can add individual files or directories to the list, or can recursively add
    the contents of a directory.  The class also allows for up-front exclusions
    in several forms (all files, all directories, all items matching a pattern,
-   or all directories containing a specific "ignore file").  Symbolic links are
-   typically backed up non-recursively, i.e. the link to a directory is backed
-   up, but not the contents of that link (we don't want to deal with recursive
-   loops, etc.).
+   all items whose basename matches a pattern, or all directories containing a
+   specific "ignore file").  Symbolic links are typically backed up
+   non-recursively, i.e. the link to a directory is backed up, but not the
+   contents of that link (we don't want to deal with recursive loops, etc.).
 
    The custom methods such as L{addFile} will only add items if they exist on
    the filesystem and do not match any exclusions that are already in place.
@@ -105,7 +106,8 @@ class FilesystemList(list):
 
    @note: Regular expression patterns that apply to paths are assumed to be
    bounded at front and back by the beginning and end of the string, i.e. they
-   are treated as if they begin with C{^} and end with C{$}.
+   are treated as if they begin with C{^} and end with C{$}.  This is true
+   whether we are matching a complete path or a basename.
 
    @note: Some platforms, like Windows, do not support soft links.  On those
    platforms, the ignore-soft-links flag can be set, but it won't do any good
@@ -114,7 +116,7 @@ class FilesystemList(list):
    @sort: __init__, addFile, addDir, addDirContents, removeFiles, removeDirs,
           removeLinks, removeMatch, removeInvalid, normalize, validate, 
           excludeFiles, excludeDirs, excludeLinks, excludePaths, 
-          excludePatterns, ignoreFile 
+          excludePatterns, excludeBasenamePatterns, ignoreFile 
    """
 
 
@@ -130,12 +132,14 @@ class FilesystemList(list):
       self._excludeLinks = False
       self._excludePaths = None
       self._excludePatterns = None
+      self._excludeBasenamePatterns = None
       self._ignoreFile = None
       self.excludeFiles = False
       self.excludeLinks = False
       self.excludeDirs = False
       self.excludePaths = []
       self.excludePatterns = []
+      self.excludeBasenamePatterns = []
       self.ignoreFile = None
 
 
@@ -223,6 +227,21 @@ class FilesystemList(list):
       """
       return self._excludePatterns
 
+   def _setExcludeBasenamePatterns(self, value):
+      """
+      Property target used to set the exclude basename patterns list.
+      A C{None} value is converted to an empty list.
+      """
+      self._excludeBasenamePatterns = []
+      if value is not None:
+         self._excludeBasenamePatterns.extend(value)
+
+   def _getExcludeBasenamePatterns(self):
+      """
+      Property target used to get the exclude basename patterns list.
+      """
+      return self._excludeBasenamePatterns
+
    def _setIgnoreFile(self, value):
       """
       Property target used to set the ignore file.
@@ -244,7 +263,10 @@ class FilesystemList(list):
    excludeDirs = property(_getExcludeDirs, _setExcludeDirs, None, "Boolean indicating whether directories should be excluded.")
    excludeLinks = property(_getExcludeLinks, _setExcludeLinks, None, "Boolean indicating whether soft links should be excluded.")
    excludePaths = property(_getExcludePaths, _setExcludePaths, None, "List of absolute paths to be excluded.")
-   excludePatterns = property(_getExcludePatterns, _setExcludePatterns, None, "List of regular expression patterns to be excluded.")
+   excludePatterns = property(_getExcludePatterns, _setExcludePatterns, None, 
+                              "List of regular expression patterns (matching complete path) to be excluded.")
+   excludeBasenamePatterns = property(_getExcludeBasenamePatterns, _setExcludeBasenamePatterns, 
+                                      None, "List of regular expression patterns (matching basename) to be excluded.")
    ignoreFile = property(_getIgnoreFile, _setIgnoreFile, None, "Name of file which will cause directory contents to be ignored.")
    
 
@@ -284,6 +306,10 @@ class FilesystemList(list):
          if re.compile(r"^%s$" % pattern).match(path):
             logger.debug("Path [%s] is excluded based on pattern [%s]." % (path, pattern))
             return 0
+      for pattern in self.excludeBasenamePatterns:
+         if re.compile(r"^%s$" % pattern).match(os.path.basename(path)):
+            logger.debug("Path [%s] is excluded based on basename pattern [%s]." % (path, pattern))
+            return 0
       self.append(path)
       logger.debug("Added file to list: [%s]" % path)
       return 1
@@ -323,11 +349,15 @@ class FilesystemList(list):
          if re.compile(r"^%s$" % pattern).match(path):
             logger.debug("Path [%s] is excluded based on pattern [%s]." % (path, pattern))
             return 0
+      for pattern in self.excludeBasenamePatterns:
+         if re.compile(r"^%s$" % pattern).match(os.path.basename(path)):
+            logger.debug("Path [%s] is excluded based on basename pattern [%s]." % (path, pattern))
+            return 0
       self.append(path)
       logger.debug("Added directory to list: [%s]" % path)
       return 1
 
-   def addDirContents(self, path, recursive=True):
+   def addDirContents(self, path, recursive=True, addSelf=True):
       """
       Adds the contents of a directory to the list.
 
@@ -347,7 +377,7 @@ class FilesystemList(list):
       only be added by name, not recursively.   Any invalid soft links (i.e.
       soft links that point to non-existent items) will be silently ignored.
 
-      @note: The L{excludeDirs} flag only controls whether any given soft link
+      @note: The L{excludeDirs} flag only controls whether any given directory
       path itself is added to the list once it has been discovered.  It does
       I{not} modify any behavior related to directory recursion.
    
@@ -357,6 +387,9 @@ class FilesystemList(list):
       @param recursive: Indicates whether directory contents should be added recursively.
       @type recursive: Boolean value
 
+      @param addSelf: Indicates whether the directory itself should be added to the list.
+      @type addSelf: Boolean value
+
       @return: Number of items recursively added to the list
 
       @raise ValueError: If path is not a directory or does not exist.
@@ -364,7 +397,7 @@ class FilesystemList(list):
       """
       path = encodePath(path)
       path = normalizeDir(path)
-      return self._addDirContentsInternal(path, recursive=recursive)
+      return self._addDirContentsInternal(path, recursive=recursive, includePath=addSelf)
 
    def _addDirContentsInternal(self, path, includePath=True, recursive=True):
       """
@@ -397,6 +430,10 @@ class FilesystemList(list):
          if re.compile(r"^%s$" % pattern).match(path):
             logger.debug("Path [%s] is excluded based on pattern [%s]." % (path, pattern))
             return added
+      for pattern in self.excludeBasenamePatterns:
+         if re.compile(r"^%s$" % pattern).match(os.path.basename(path)):
+            logger.debug("Path [%s] is excluded based on basename pattern [%s]." % (path, pattern))
+            return added
       if self.ignoreFile is not None and os.path.exists(os.path.join(path, self.ignoreFile)):
          logger.debug("Path [%s] is excluded based on ignore file." % path)
          return added
@@ -540,7 +577,12 @@ class FilesystemList(list):
       is faster to call this method than to call the L{removeFiles},
       L{removeDirs} or L{removeLinks} methods individually.  If you know which
       patterns you will want to remove ahead of time, you may be better off
-      setting L{excludePatterns} before adding items to the list.
+      setting L{excludePatterns} or L{excludeBasenamePatterns} before adding
+      items to the list.
+
+      @note: Unlike when using the exclude lists, the pattern here is I{not}
+      bounded at the front and the back of the string.  You can use any pattern
+      you want.
 
       @param pattern: Regular expression pattern representing entries to remove
 
@@ -603,6 +645,27 @@ class FilesystemList(list):
       return True
 
 
+########################################################################
+# SpanItem class definition
+########################################################################
+
+class SpanItem(object):
+   """
+   Item returned by L{BackupFileList.generateSpan}.
+   """
+   def __init__(self, fileList, size, capacity, utilization):
+      """
+      Create object.
+      @param fileList: List of files
+      @param size: Size (in bytes) of files
+      @param utilization: Utilization, as a percentage (0-100)
+      """
+      self.fileList = fileList
+      self.size = size
+      self.capacity = capacity
+      self.utilization = utilization
+
+
 ########################################################################
 # BackupFileList class definition
 ########################################################################
@@ -808,17 +871,89 @@ class BackupFileList(FilesystemList):
       @return: Copy of list with total size no larger than indicated capacity
       @raise ValueError: If the algorithm is invalid.
       """
+      table = self._getKnapsackTable()
+      function = BackupFileList._getKnapsackFunction(algorithm)
+      return function(table, capacity)[0]
+
+   def generateSpan(self, capacity, algorithm="worst_fit"):
+      """
+      Splits the list of items into sub-lists that fit in a given capacity.
+
+      Sometimes, callers need split to a backup file list into a set of smaller
+      lists.  For instance, you could use this to "span" the files across a set
+      of discs.
+
+      The fitting is done using the functions in the knapsack module.  By
+      default, the first fit algorithm is used, but you can also choose
+      from best fit, worst fit and alternate fit.
+
+      @note: If any of your items are larger than the capacity, then it won't
+      be possible to find a solution.  In this case, a value error will be
+      raised.
+
+      @param capacity: Maximum capacity among the files in the new list
+      @type capacity: Integer, in bytes
+
+      @param algorithm: Knapsack (fit) algorithm to use
+      @type algorithm: One of "first_fit", "best_fit", "worst_fit", "alternate_fit"
+
+      @return: List of L{SpanItem} objects.
+
+      @raise ValueError: If the algorithm is invalid.
+      @raise ValueError: If it's not possible to fit some items
+      """
+      spanItems = []
+      function = BackupFileList._getKnapsackFunction(algorithm)
+      table = self._getKnapsackTable(capacity)
+      iteration = 0
+      while len(table) > 0:
+         iteration += 1
+         fit = function(table, capacity)
+         if len(fit[0]) == 0:
+            # Should never happen due to validations in _convertToKnapsackForm(), but let's be safe
+            raise ValueError("After iteration %d, unable to add any new items." % iteration)
+         removeKeys(table, fit[0])
+         utilization = (float(fit[1])/float(capacity))*100.0
+         item = SpanItem(fit[0], fit[1], capacity, utilization)
+         spanItems.append(item)
+      return spanItems
+
+   def _getKnapsackTable(self, capacity=None):
+      """
+      Converts the list into the form needed by the knapsack algorithms.
+      @return: Dictionary mapping file name to tuple of (file path, file size).
+      """
       table = { }
       for entry in self:
          if os.path.islink(entry):
             table[entry] = (entry, 0.0)
          elif os.path.isfile(entry):
-            table[entry] = (entry, float(os.stat(entry).st_size))
-      if algorithm == "first_fit": return firstFit(table, capacity)[0]
-      elif algorithm == "best_fit": return bestFit(table, capacity)[0]
-      elif algorithm == "worst_fit": return worstFit(table, capacity)[0]
-      elif algorithm == "alternate_fit": return alternateFit(table, capacity)[0]
-      else: raise ValueError("Algorithm [%s] is invalid." % algorithm);
+            size = float(os.stat(entry).st_size)
+            if capacity is not None:
+               if size > capacity:
+                  raise ValueError("File [%s] cannot fit in capacity %s." % (entry, displayBytes(capacity)))
+            table[entry] = (entry, size)
+      return table
+
+   def _getKnapsackFunction(algorithm):
+      """
+      Returns a reference to the function associated with an algorithm name.
+      Algorithm name must be one of "first_fit", "best_fit", "worst_fit", "alternate_fit"
+      @param algorithm: Name of the algorithm
+      @return: Reference to knapsack function
+      @raise ValueError: If the algorithm name is unknown.
+      """
+      if algorithm == "first_fit":
+         return firstFit
+      elif algorithm == "best_fit":
+         return bestFit
+      elif algorithm == "worst_fit":
+         return worstFit
+      elif algorithm == "alternate_fit":
+         return alternateFit
+      else:
+         raise ValueError("Algorithm [%s] is invalid." % algorithm);
+   _getKnapsackFunction = staticmethod(_getKnapsackFunction)
 
    def generateTarfile(self, path, mode='tar', ignore=False, flat=False):
       """
@@ -1042,7 +1177,7 @@ class PurgeItemList(FilesystemList):
    # Add methods
    ##############
 
-   def addDirContents(self, path, recursive=True):
+   def addDirContents(self, path, recursive=True, addSelf=False):
       """
       Adds the contents of a directory to the list.
 
@@ -1075,6 +1210,8 @@ class PurgeItemList(FilesystemList):
 
       @param recursive: Indicates whether directory contents should be added recursively.
       @type recursive: Boolean value
+   
+      @param addSelf: Ignored in this subclass.
 
       @return: Number of items recursively added to the list
 
@@ -1223,7 +1360,7 @@ def compareContents(path1, path2, verbose=False):
    @type path1: String representing a path on disk
 
    @param path2: First path to compare.
-   @type path12 String representing a path on disk
+   @type path2: String representing a path on disk
 
    @param verbose: Indicates whether a verbose response should be given.
    @type verbose: Boolean
@@ -1239,18 +1376,35 @@ def compareContents(path1, path2, verbose=False):
       path2List = BackupFileList()
       path2List.addDirContents(path2)
       path2Digest = path2List.generateDigestMap(stripPrefix=normalizeDir(path2))
-      if not verbose:
-         if path1Digest != path2Digest:
-            raise ValueError("Consistency check failed.")
-      else:
-         list1 = UnorderedList(path1Digest.keys())
-         list2 = UnorderedList(path2Digest.keys())
-         if list1 != list2:
-            raise ValueError("Directories contain a different set of files.")
-         for key in list1:
-            if path1Digest[key] != path2Digest[key]:
-               raise ValueError("File contents for [%s] vary between directories." % key)
+      compareDigestMaps(path1Digest, path2Digest, verbose)
    except IOError, e:
       logger.error("I/O error encountered during consistency check.")
       raise e
 
+def compareDigestMaps(digest1, digest2, verbose=False):
+   """
+   Compares two digest maps and throws an exception if they differ.
+
+   @param digest1: First digest to compare.
+   @type digest1: Digest as returned from BackupFileList.generateDigestMap()
+
+   @param digest2: Second digest to compare.
+   @type digest2: Digest as returned from BackupFileList.generateDigestMap()
+
+   @param verbose: Indicates whether a verbose response should be given.
+   @type verbose: Boolean
+
+   @raise ValueError: If the two directories are not equivalent.
+   """
+   if not verbose:
+      if digest1 != digest2:
+         raise ValueError("Consistency check failed.")
+   else:
+      list1 = UnorderedList(digest1.keys())
+      list2 = UnorderedList(digest2.keys())
+      if list1 != list2:
+         raise ValueError("Directories contain a different set of files.")
+      for key in list1:
+         if digest1[key] != digest2[key]:
+            raise ValueError("File contents for [%s] vary between directories." % key)
+
