@@ -11,7 +11,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: release.py 1022 2006-12-19 04:53:11Z pronovic $
+# Revision : $Id: release.py 1194 2007-03-30 01:57:33Z pronovic $
 # Purpose  : Provides location to maintain release information.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -31,10 +31,10 @@ Provides location to maintain version information.
 @author: Kenneth J. Pronovici <pronovic@ieee.org>
 """
 
-AUTHOR      = "Kenneth J. Pronovici, et. al"
+AUTHOR      = "Kenneth J. Pronovici"
 EMAIL       = "pronovic@ieee.org"
-COPYRIGHT   = "2004-2006"
-VERSION     = "2.9.0"
-DATE        = "18 Dec 2006"
+COPYRIGHT   = "2004-2007"
+VERSION     = "2.13.1"
+DATE        = "29 Mar 2007"
 URL         = "http://cedar-solutions.com/software/cedar-backup"
 
