@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: testutil.py 983 2006-06-24 18:27:10Z pronovic $
+# Revision : $Id: testutil.py 1185 2007-03-25 19:04:50Z pronovic $
 # Purpose  : Provides unit-testing utilities.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -72,6 +72,8 @@ import getpass
 import random
 import string
 import platform
+import logging
+from StringIO import StringIO
 
 from CedarBackup2.util import encodePath
 
@@ -80,6 +82,29 @@ from CedarBackup2.util import encodePath
 # Public functions
 ########################################################################
 
+##############################
+# setupDebugLogger() function
+##############################
+
+def setupDebugLogger():
+   """
+   Sets up a screen logger for debugging purposes.
+
+   Normally, the CLI functionality configures the logger so that
+   things get written to the right place.  However, for debugging
+   it's sometimes nice to just get everything -- debug information
+   and output -- dumped to the screen.  This function takes care
+   of that.
+   """
+   logger = logging.getLogger("CedarBackup2")
+   logger.setLevel(logging.DEBUG)    # let the logger see all messages
+   formatter = logging.Formatter(fmt="%(message)s")
+   handler = logging.StreamHandler(strm=sys.stdout)
+   handler.setFormatter(formatter)
+   handler.setLevel(logging.DEBUG)
+   logger.addHandler(handler)
+
+
 ###########################
 # findResources() function
 ###########################
@@ -308,6 +333,35 @@ def failUnlessAssignRaises(testCase, exception, object, property, value):
       testCase.fail("Expected assignment to raise %s, but got %s instead." % (ValueError, instead.__class__.__name__))
 
 
+###########################
+# captureOutput() function
+###########################
+
+def captureOutput(callable):
+   """
+   Captures the output (stdout, stderr) of a function or a method.
+
+   Some of our functions don't do anything other than just print output.  We
+   need a way to test these functions (at least nominally) but we don't want
+   any of the output spoiling the test suite output.
+
+   This function just creates a dummy file descriptor that can be used as a
+   target by the callable function, rather than C{stdout} or C{stderr}.
+
+   @note: This method assumes that C{callable} doesn't take any arguments
+   besides keyword argument C{fd} to specify the file descriptor.
+
+   @param callable: Callable function or method.
+
+   @return: Output of function, as one big string.
+   """
+   fd = StringIO()
+   callable(fd=fd)
+   result = fd.getvalue()
+   fd.close()
+   return result
+
+
 #########################
 # _isPlatform() function
 #########################
