@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2004-2006 Kenneth J. Pronovici.
+# Copyright (c) 2004-2007 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # Portions copyright (c) 2001, 2002 Python Software Foundation.
@@ -30,7 +30,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: util.py 956 2006-05-26 23:05:12Z pronovic $
+# Revision : $Id: util.py 1172 2007-03-09 20:34:37Z pronovic $
 # Purpose  : Provides general-purpose utilities.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -42,13 +42,15 @@
 """
 Provides general-purpose utilities. 
 
-@sort: AbsolutePathList, ObjectTypeList, PathResolverSingleton, 
-       convertSize, getUidGid, changeOwnership, splitCommandLine, 
+@sort: AbsolutePathList, ObjectTypeList, RestrictedValueList, RegexMatchList,
+       _Vertex, DirectedGraph, PathResolverSingleton, 
+       sortDict, convertSize, getUidGid, changeOwnership, splitCommandLine,
        resolveCommand, executeCommand, calculateFileAge, encodePath, nullDevice,
+       deriveDayOfWeek, isStartOfWeek, buildNormalizedPath, 
        ISO_SECTOR_SIZE, BYTES_PER_SECTOR, 
        BYTES_PER_KBYTE, BYTES_PER_MBYTE, BYTES_PER_GBYTE, KBYTES_PER_MBYTE, MBYTES_PER_GBYTE, 
        SECONDS_PER_MINUTE, MINUTES_PER_HOUR, HOURS_PER_DAY, SECONDS_PER_DAY, 
-       UNIT_BYTES, UNIT_KBYTES, UNIT_MBYTES, UNIT_SECTORS
+       UNIT_BYTES, UNIT_KBYTES, UNIT_MBYTES, UNIT_GBYTES, UNIT_SECTORS
 
 @var ISO_SECTOR_SIZE: Size of an ISO image sector, in bytes.
 @var BYTES_PER_SECTOR: Number of bytes (B) per ISO sector.
@@ -64,6 +66,7 @@ Provides general-purpose utilities.
 @var UNIT_BYTES: Constant representing the byte (B) unit for conversion.
 @var UNIT_KBYTES: Constant representing the kilobyte (kB) unit for conversion.
 @var UNIT_MBYTES: Constant representing the megabyte (MB) unit for conversion.
+@var UNIT_GBYTES: Constant representing the gigabyte (GB) unit for conversion.
 @var UNIT_SECTORS: Constant representing the ISO sector unit for conversion.
 
 @author: Kenneth J. Pronovici <pronovic@ieee.org>
@@ -124,6 +127,7 @@ SECONDS_PER_DAY    = SECONDS_PER_MINUTE * MINUTES_PER_HOUR * HOURS_PER_DAY
 UNIT_BYTES         = 0
 UNIT_KBYTES        = 1
 UNIT_MBYTES        = 2
+UNIT_GBYTES        = 4
 UNIT_SECTORS       = 3
 
 MTAB_FILE          = "/etc/mtab"
@@ -376,6 +380,13 @@ class RestrictedContentList(UnorderedList):
 
    The C{valuesDescr} value will be used in exceptions, i.e. C{"Item must be
    one of values in VALID_ACTIONS"} if C{valuesDescr} is C{"VALID_ACTIONS"}.
+
+   @note:  This class doesn't make any attempt to trap for nonsensical
+   arguments.  All of the values in the values list should be of the same type
+   (i.e. strings).  Then, all list operations also need to be of that type
+   (i.e. you should always insert or append just strings).  If you mix types --
+   for instance lists and strings -- you will likely see AttributeError
+   exceptions or other problems.
    """
    
    def __init__(self, valuesList, valuesDescr):
@@ -417,10 +428,264 @@ class RestrictedContentList(UnorderedList):
 
 
 ########################################################################
-# PathResolverSingleton class definition
+# RegexMatchList class definition
 ########################################################################
 
-class PathResolverSingleton:
+class RegexMatchList(UnorderedList):
+
+   """
+   Class representing a list containing only strings that match a regular expression.
+
+   If C{emptyAllowed} is passed in as C{False}, then empty strings are
+   explicitly disallowed, even if they happen to match the regular expression.
+   (C{None} values are always disallowed, since string operations are not
+   permitted on C{None}.)
+
+   This is an unordered list.
+
+   We override the C{append}, C{insert} and C{extend} methods to ensure that
+   any item added to the list matches the indicated regular expression.  
+
+   @note: If you try to put values that are not strings into the list, you will
+   likely get either TypeError or AttributeError exceptions as a result.
+   """
+   
+   def __init__(self, valuesRegex, emptyAllowed=True):
+      """
+      Initializes a list restricted to containing certain values.
+      @param valuesRegex: Regular expression that must be matched, as a string
+      @param emptyAllowed: Indicates whether empty or None values are allowed.
+      """
+      self.valuesRegex = valuesRegex
+      self.emptyAllowed = emptyAllowed
+      self.pattern = re.compile(self.valuesRegex)
+
+   def append(self, item):
+      """
+      Overrides the standard C{append} method.
+      @raise ValueError: If item is None
+      @raise ValueError: If item is empty and empty values are not allowed
+      @raise ValueError: If item does not match the configured regular expression
+      """
+      if item is None or (not self.emptyAllowed and item == ""):
+         raise ValueError("Item must be non-empty.")
+      if not self.pattern.search(item):
+         raise ValueError("Item must match regular expression [%s]." % self.valuesRegex)
+      list.append(self, item)
+
+   def insert(self, index, item):
+      """
+      Overrides the standard C{insert} method.
+      @raise ValueError: If item is None
+      @raise ValueError: If item is empty and empty values are not allowed
+      @raise ValueError: If item does not match the configured regular expression
+      """
+      if item is None or (not self.emptyAllowed and item == ""):
+         raise ValueError("Item must be non-empty.")
+      if not self.pattern.search(item):
+         raise ValueError("Item must match regular expression [%s]." % self.valuesRegex)
+      list.insert(self, index, item)
+
+   def extend(self, seq):
+      """
+      Overrides the standard C{insert} method.
+      @raise ValueError: If any item is None
+      @raise ValueError: If any item is empty and empty values are not allowed
+      @raise ValueError: If any item does not match the configured regular expression
+      """
+      for item in seq:
+         if item is None or (not self.emptyAllowed and item == ""):
+            raise ValueError("Item must be non-empty.")
+         if not self.pattern.search(item):
+            raise ValueError("Item must match regular expression [%s]." % self.valuesRegex)
+      list.extend(self, seq)
+
+
+########################################################################
+# Directed graph implementation
+########################################################################
+
+class _Vertex(object):
+
+   """
+   Represents a vertex (or node) in a directed graph.
+   """
+
+   def __init__(self, name):
+      """
+      Constructor.
+      @param name: Name of this graph vertex.
+      @type name: String value.
+      """
+      self.name = name
+      self.endpoints = []
+      self.state = None
+
+class DirectedGraph(object):
+
+   """
+   Represents a directed graph.
+
+   A graph B{G=(V,E)} consists of a set of vertices B{V} together with a set
+   B{E} of vertex pairs or edges.  In a directed graph, each edge also has an
+   associated direction (from vertext B{v1} to vertex B{v2}).  A C{DirectedGraph} 
+   object provides a way to construct a directed graph and execute a depth-
+   first search.
+
+   This data structure was designed based on the graphing chapter in
+   U{The Algorithm Design Manual<http://www2.toki.or.id/book/AlgDesignManual/>},
+   by Steven S. Skiena.  
+
+   This class is intended to be used by Cedar Backup for dependency ordering.
+   Because of this, it's not quite general-purpose.  Unlike a "general" graph,
+   every vertex in this graph has at least one edge pointing to it, from a
+   special "start" vertex.  This is so no vertices get "lost" either because
+   they have no dependencies or because nothing depends on them.
+   """
+
+   _UNDISCOVERED = 0
+   _DISCOVERED   = 1
+   _EXPLORED     = 2
+
+   def __init__(self, name):
+      """
+      Directed graph constructor.
+
+      @param name: Name of this graph.
+      @type name: String value.
+      """
+      if name is None or name == "":
+         raise ValueError("Graph name must be non-empty.")
+      self._name = name
+      self._vertices = {}
+      self._startVertex = _Vertex(None)  # start vertex is only vertex with no name
+
+   def __repr__(self):
+      """
+      Official string representation for class instance.
+      """
+      return "DirectedGraph(%s)" % self.name
+
+   def __str__(self):
+      """
+      Informal string representation for class instance.
+      """
+      return self.__repr__()
+
+   def __cmp__(self, other):
+      """
+      Definition of equals operator for this class.
+      @param other: Other object to compare to.
+      @return: -1/0/1 depending on whether self is C{<}, C{=} or C{>} other.
+      """
+      if other is None:
+         return 1
+      if self._name != other._name:
+         if self._name < other._name:
+            return -1
+         else:
+            return 1
+      if self._vertices != other._vertices:
+         if self._vertices < other._vertices:
+            return -1
+         else:
+            return 1
+      return 0
+
+   def _getName(self):
+      """
+      Property target used to get the graph name.
+      """
+      return self._name
+
+   name = property(_getName, None, None, "Name of the graph.")
+
+   def createVertex(self, name):
+      """
+      Creates a named vertex.
+      @param name: vertex name
+      @raise ValueError: If the vertex name is C{None} or empty.
+      """
+      if name is None or name == "":
+         raise ValueError("Vertex name must be non-empty.")
+      vertex = _Vertex(name)
+      self._startVertex.endpoints.append(vertex)  # so every vertex is connected at least once
+      self._vertices[name] = vertex
+
+   def createEdge(self, start, finish):
+      """
+      Adds an edge with an associated direction, from C{start} vertex to C{finish} vertex.
+      @param start: Name of start vertex.
+      @param finish: Name of finish vertex.
+      @raise ValueError: If one of the named vertices is unknown.
+      """
+      try:
+         startVertex = self._vertices[start]
+         finishVertex = self._vertices[finish]
+         startVertex.endpoints.append(finishVertex)
+      except KeyError, e:
+         raise ValueError("Vertex [%s] could not be found." % e)
+
+   def topologicalSort(self):
+      """
+      Implements a topological sort of the graph.
+
+      This method also enforces that the graph is a directed acyclic graph,
+      which is a requirement of a topological sort.
+
+      A directed acyclic graph (or "DAG") is a directed graph with no directed
+      cycles.  A topological sort of a DAG is an ordering on the vertices such
+      that all edges go from left to right.  Only an acyclic graph can have a
+      topological sort, but any DAG has at least one topological sort.  
+
+      Since a topological sort only makes sense for an acyclic graph, this 
+      method throws an exception if a cycle is found.
+
+      A depth-first search only makes sense if the graph is acyclic.  If the
+      graph contains any cycles, it is not possible to determine a consistent
+      ordering for the vertices.
+
+      @note: If a particular vertex has no edges, then its position in the
+      final list depends on the order in which the vertices were created in the
+      graph.  If you're using this method to determine a dependency order, this
+      makes sense: a vertex with no dependencies can go anywhere (and will).
+
+      @return: Ordering on the vertices so that all edges go from left to right.
+
+      @raise ValueError: If a cycle is found in the graph.
+      """
+      ordering = []
+      for key in self._vertices:
+         vertex = self._vertices[key]
+         vertex.state = self._UNDISCOVERED
+      for key in self._vertices:
+         vertex = self._vertices[key]
+         if vertex.state == self._UNDISCOVERED:
+            self._topologicalSort(self._startVertex, ordering)
+      return ordering
+
+   def _topologicalSort(self, vertex, ordering):
+      """
+      Recursive depth first search function implementing topological sort.
+      @param vertex: Vertex to search
+      @param ordering: List of vertices in proper order
+      """
+      vertex.state = self._DISCOVERED
+      for endpoint in vertex.endpoints:
+         if endpoint.state == self._UNDISCOVERED:
+            self._topologicalSort(endpoint, ordering)
+         elif endpoint.state != self._EXPLORED:
+            raise ValueError("Cycle found in graph (found '%s' while searching '%s')." % (vertex.name, endpoint.name))
+      if vertex.name is not None:
+         ordering.insert(0, vertex.name) 
+      vertex.state = self._EXPLORED
+
+
+########################################################################
+# PathResolverSingleton class defkinition
+########################################################################
+
+class PathResolverSingleton(object):
 
    """
    Singleton used for resolving executable paths.
@@ -612,9 +877,42 @@ else: # _PIPE_IMPLEMENTATION == "popen2.Popen4"
 
 
 ########################################################################
-# Public functions
+# General utility functions
 ########################################################################
 
+######################
+# sortDict() function
+######################
+
+def sortDict(d):
+   """
+   Returns the keys of the dictionary sorted by value.
+   There are cuter ways to do this in Python 2.4, but we're compatible with 2.3.
+   @param d: Dictionary to operate on
+   @return: List of dictionary keys sorted in order by dictionary value.
+   """
+   items = d.items()
+   items.sort(lambda x, y: cmp(x[1], y[1]))
+   return [key for key, value in items]
+
+
+########################
+# removeKeys() function
+########################
+
+def removeKeys(d, keys):
+   """
+   Removes all of the keys from the dictionary.
+   The dictionary is altered in-place. 
+   Each key must exist in the dictionary.
+   @param d: Dictionary to operate on
+   @param keys: List of keys to remove
+   @raise KeyError: If one of the keys does not exist
+   """
+   for key in keys:
+      del d[key]
+
+
 #########################
 # convertSize() function
 #########################
@@ -630,9 +928,10 @@ def convertSize(size, fromUnit, toUnit):
    The available units are:
 
       - C{UNIT_BYTES} - Bytes
-      - C{UNIT_KBYTES} - Kilobytes, where 1kB = 1024B
-      - C{UNIT_MBYTES} - Megabytes, where 1MB = 1024kB
-      - C{UNIT_SECTORS} - Sectors, where 1 sector = 2048B
+      - C{UNIT_KBYTES} - Kilobytes, where 1 kB = 1024 B
+      - C{UNIT_MBYTES} - Megabytes, where 1 MB = 1024 kB
+      - C{UNIT_GBYTES} - Gigabytes, where 1 GB = 1024 MB
+      - C{UNIT_SECTORS} - Sectors, where 1 sector = 2048 B
 
    @param size: Size to convert
    @type size: Integer or float value in units of C{fromUnit} 
@@ -646,26 +945,32 @@ def convertSize(size, fromUnit, toUnit):
    @return: Number converted to new unit, as a float.
    @raise ValueError: If one of the units is invalid.
    """
+   if size is None:
+      raise ValueError("Cannot convert size of None.")
    if fromUnit == UNIT_BYTES:
       byteSize = float(size)
    elif fromUnit == UNIT_KBYTES:
       byteSize = float(size) * BYTES_PER_KBYTE
    elif fromUnit == UNIT_MBYTES:
       byteSize = float(size) * BYTES_PER_MBYTE
+   elif fromUnit == UNIT_GBYTES:
+      byteSize = float(size) * BYTES_PER_GBYTE
    elif fromUnit == UNIT_SECTORS:
       byteSize = float(size) * BYTES_PER_SECTOR
    else:
-      raise ValueError("Unknown 'from' unit %d." % fromUnit)
+      raise ValueError("Unknown 'from' unit %s." % fromUnit)
    if toUnit == UNIT_BYTES:
       return byteSize
    elif toUnit == UNIT_KBYTES:
       return byteSize / BYTES_PER_KBYTE
    elif toUnit == UNIT_MBYTES:
       return byteSize / BYTES_PER_MBYTE
+   elif toUnit == UNIT_GBYTES:
+      return byteSize / BYTES_PER_GBYTE
    elif toUnit == UNIT_SECTORS:
       return byteSize / BYTES_PER_SECTOR
    else:
-      raise ValueError("Unknown 'to' unit %d." % toUnit)
+      raise ValueError("Unknown 'to' unit %s." % toUnit)
 
 
 ##########################
@@ -678,7 +983,7 @@ def displayBytes(bytes, digits=2):
 
    It's rather difficult to look at a number like "72372224 bytes" and get any
    meaningful information out of it.  It would be more useful to see something
-   like "72.37 MB".  That's what this function does.  Any time you want to display
+   like "69.02 MB".  That's what this function does.  Any time you want to display
    a byte value, i.e.::
 
       print "Size: %s bytes" % bytes
@@ -689,7 +994,9 @@ def displayBytes(bytes, digits=2):
 
    What comes out will be sensibly formatted.  The indicated number of digits
    will be listed after the decimal point, rounded based on whatever rules are
-   used by Python's standard C{%f} string format specifier.
+   used by Python's standard C{%f} string format specifier. (Values less than 1
+   kB will be listed in bytes and will not have a decimal point, since the
+   concept of a fractional byte is nonsensical.)
 
    @param bytes: Byte quantity.
    @type bytes: Integer number of bytes.
@@ -699,6 +1006,8 @@ def displayBytes(bytes, digits=2):
 
    @return: String, formatted for sensible display.
    """
+   if(bytes is None):
+      raise ValueError("Cannot display byte value of None.")
    bytes = float(bytes)
    if math.fabs(bytes) < BYTES_PER_KBYTE:
       format = "%.0f bytes"
@@ -814,13 +1123,18 @@ def getUidGid(user, group):
 def changeOwnership(path, user, group):
    """
    Changes ownership of path to match the user and group.
-   This is a no-op if user/group functionality is not available on the platform.
+
+   This is a no-op if user/group functionality is not available on the
+   platform, or if the either passed-in user or group is C{None}.
+
    @param path: Path whose ownership to change.
    @param user: User which owns file.
    @param group: Group which owns file.
    """
    if _UID_GID_AVAILABLE:
-      if os.getuid() != 0:
+      if user is None or group is None:
+         logger.debug("User or group is None, so not attempting to change owner on [%s]." % path)
+      elif os.getuid() != 0:
          logger.debug("Not root, so not attempting to change owner on [%s]." % path)
       else:
          try:
@@ -852,7 +1166,11 @@ def splitCommandLine(commandLine):
    @type commandLine: String, i.e. "cback --verbose stage store"
 
    @return: List of arguments, suitable for passing to L{popen2}.
+
+   @raise ValueError: If the command line is None.
    """
+   if commandLine is None:
+      raise ValueError("Cannot split command line of None.")
    fields = re.findall('[^ "]+|"[^"]+"', commandLine)
    fields = map(lambda field: field.replace('"', ''), fields)
    return fields
@@ -923,7 +1241,7 @@ def executeCommand(command, args, returnOutput=False, ignoreStderr=False, doNotL
    the L{popen2.Popen4} documentation.  If C{returnOutput} is passed in as
    C{True}, the function will return a tuple of C{(status, output)} where
    C{output} is a list of strings, one entry per line in the output from the
-   command.  Output is always logged to the C{ouputLogger.info()} target,
+   command.  Output is always logged to the C{outputLogger.info()} target,
    regardless of whether it's returned.
 
    By default, C{stdout} and C{stderr} will be intermingled in the output.
@@ -1295,3 +1613,109 @@ def nullDevice():
       else:
          return "/dev/null"
 
+
+##############################
+# deriveDayOfWeek() function
+##############################
+
+def deriveDayOfWeek(dayName):
+   """
+   Converts English day name to numeric day of week as from C{time.localtime}.
+
+   For instance, the day C{monday} would be converted to the number C{0}.
+
+   @param dayName: Day of week to convert
+   @type dayName: string, i.e. C{"monday"}, C{"tuesday"}, etc.
+
+   @returns: Integer, where Monday is 0 and Sunday is 6; or -1 if no conversion is possible.
+   """
+   if dayName.lower() == "monday":
+      return 0
+   elif dayName.lower() == "tuesday":
+      return 1
+   elif dayName.lower() == "wednesday":
+      return 2
+   elif dayName.lower() == "thursday":
+      return 3
+   elif dayName.lower() == "friday":
+      return 4
+   elif dayName.lower() == "saturday":
+      return 5
+   elif dayName.lower() == "sunday":
+      return 6
+   else:
+      return -1  # What else can we do??  Thrown an exception, I guess.
+
+
+###########################
+# isStartOfWeek() function
+###########################
+
+def isStartOfWeek(startingDay):
+   """
+   Indicates whether "today" is the backup starting day per configuration.
+
+   If the current day's English name matches the indicated starting day, then
+   today is a starting day.
+
+   @param startingDay: Configured starting day.
+   @type startingDay: string, i.e. C{"monday"}, C{"tuesday"}, etc.
+
+   @return: Boolean indicating whether today is the starting day.
+   """
+   value = time.localtime().tm_wday == deriveDayOfWeek(startingDay)
+   if value:
+      logger.debug("Today is the start of the week.")
+   else:
+      logger.debug("Today is NOT the start of the week.")
+   return value
+
+
+#################################
+# buildNormalizedPath() function
+#################################
+
+def buildNormalizedPath(path):
+   """
+   Returns a "normalized" path based on a path name.
+
+   A normalized path is a representation of a path that is also a valid file
+   name.  To make a valid file name out of a complete path, we have to convert
+   or remove some characters that are significant to the filesystem -- in
+   particular, the path separator and any leading C{'.'} character (which would
+   cause the file to be hidden in a file listing).  
+
+   Note that this is a one-way transformation -- you can't safely derive the
+   original path from the normalized path.
+
+   To normalize a path, we begin by looking at the first character.  If the
+   first character is C{'/'} or C{'\\'}, it gets removed.  If the first
+   character is C{'.'}, it gets converted to C{'_'}.  Then, we look through the
+   rest of the path and convert all remaining C{'/'} or C{'\\'} characters
+   C{'-'}, and all remaining whitespace characters to C{'_'}.
+
+   As a special case, a path consisting only of a single C{'/'} or C{'\\'}
+   character will be converted to C{'-'}.
+
+   @param path: Path to normalize
+
+   @return: Normalized path as described above.
+
+   @raise ValueError: If the path is None
+   """
+   if path is None:
+      raise ValueError("Cannot normalize path None.")
+   elif len(path) == 0:
+      return path
+   elif path == "/" or path == "\\":
+      return "-"
+   else:
+      normalized = path
+      normalized = re.sub(r"^\/", "", normalized)  # remove leading '/'
+      normalized = re.sub(r"^\\", "", normalized)  # remove leading '\'
+      normalized = re.sub(r"^\.", "_", normalized) # convert leading '.' to '_' so file won't be hidden
+      normalized = re.sub(r"\/", "-", normalized)  # convert all '/' characters to '-'
+      normalized = re.sub(r"\\", "-", normalized)  # convert all '\' characters to '-'
+      normalized = re.sub(r"\s", "_", normalized)  # convert all whitespace to '_'
+      return normalized
+
